#!/usr/bin/env python
#-*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html

# ------------
#| Change Log |
# ------------

# Ver. 1.0: Initial release:(19/10/2022) small piece of MrQ Actions RPL code called from 'Layers' context menu

from gimpfu import *
import random

def shuffle_layers(image,layer):

	num_layers, layer_ids = pdb.gimp_image_get_layers(image)
	layer_names_list = image.layers
	loop = 0
	shuffle = num_layers - 1
	if shuffle > 1:
		while loop < num_layers -1:
			random.seed()
			iloop = 0
			deal = random.randrange(0,num_layers -2)
			random.seed()
			pdb.gimp_image_lower_item(image, layer_names_list[deal])
			num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			layer_names_list = image.layers
			loop += 1

################################################## sort-layers.py ########################################################
# sort-layers.py by code ShadowKyogre: https://github.com/ShadowKyogre/gimp-plugins/blob/master/plug-ins/sort-layers.py

def sortLayers(image, in_rev, key_func):
	linked_candidates=list(filter(lambda x:hasattr(x, 'layers'), image.layers))
	if len(linked_candidates) > 0:
		image.undo_group_start()
		for layer in linked_candidates:
			sorted_layers=sorted(layer.layers, key=key_func, reverse=in_rev)
			for n,item in enumerate(sorted_layers):
				pdb.gimp_image_reorder_item(image, item, layer, n)
		image.undo_group_end()
		gimp.displays_flush()
		return
	elif hasattr(image.active_layer, 'layers'):
		sorted_layers=sorted(image.active_layer.layers, key=key_func, reverse=in_rev)
		parent=image.active_layer
	else:
		parent=image.active_layer.parent
		if parent is None:
			sorted_layers=sorted(image.layers, key=key_func, reverse=in_rev)
		else:
			sorted_layers=sorted(parent.children, key=key_func, reverse=in_rev)
	image.undo_group_start()
	for n,item in enumerate(sorted_layers):
		pdb.gimp_image_reorder_item(image, item, parent, n)
	image.undo_group_end()
	gimp.displays_flush()


########################################################################################################################
	
import math, glob
import sys,os,os.path,traceback
global apply_layers
import traceback
from array import array
apply_layers = []

def check_layers(image,layer,layergroup):

	if layergroup==0: # All layers
		apply_layers.append(layer)

	elif layergroup==1: # Visible layers
		if layer.visible:
			apply_layers.append(layer)
			
	elif layergroup==2: #Not visible layers
		if not layer.visible:
			apply_layers.append(layer)
			
	elif layergroup==3: # Linked layers
		if layer.linked:
			apply_layers.append(layer)
			
	elif layergroup==4: # Not-linked layers
		if not layer.linked:
			apply_layers.append(layer)
			
	elif layergroup==5: # Active only
		x = pdb.gimp_image_get_active_layer(image)
		layer_position = pdb.gimp_image_get_item_position(image,x)
		position = pdb.gimp_image_get_item_position(image,layer)
		if position == layer_position:
			apply_layers.append(layer)
				
	elif layergroup==6: # Active and Above layers
		x = pdb.gimp_image_get_active_layer(image)
		layer_position = pdb.gimp_image_get_item_position(image,x)
		position = pdb.gimp_image_get_item_position(image,layer)
		if position <= layer_position:
			apply_layers.append(layer)
				
	elif layergroup==7: # Active and Below layers
		x = pdb.gimp_image_get_active_layer(image)
		layer_position = pdb.gimp_image_get_item_position(image,x)
		position = pdb.gimp_image_get_item_position(image,layer)
		if position >= layer_position:
			apply_layers.append(layer)
							
	for child in layer.children: # recursive call to check on children here.
		check_layers(image,child,layergroup)
		

def copy_layers_RPL(image, drawable, iterations):  

	for item in range(0,int(iterations)):
		copy=pdb.gimp_layer_copy(drawable,1)
		pdb.gimp_image_add_layer(image,copy,2)

	pdb.gimp_displays_flush()

def python_fu_MrQ_Switches_Layers(image, layer, layergroup, rev):

	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	for currentlayer in image.layers:
		check_layers(image,currentlayer, layergroup)
	    
	if rev==4:	# GROUP

		group_new = pdb.gimp_layer_group_new(image)
		parent = pdb.gimp_item_get_parent(group_new)
		position = pdb.gimp_image_get_item_position(image, layer)
		pdb.gimp_image_insert_layer(image,group_new, None, 0) 
		group_new.name = "Group New"
		for layer in apply_layers:
			pdb.gimp_image_reorder_item(image, layer, group_new, position)
			pdb.gimp_image_lower_item_to_bottom(image, layer)
	if rev==17:  #  start OPEN AS LAYERS IN NEW IMAGE cd..
		image_create = pdb.gimp_image_new(image.width, image.height, 0)
		display = pdb.gimp_display_new(image_create)
		
	for layer in apply_layers:
			
		if rev==0:	# VISIBLE LAYER
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_item_set_visible(layer, 1)			
		if rev==1: # NOT-VISIBLE LAYER
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_item_set_visible(layer, 0)			
		if rev==2:	# LINKED LAYER
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_item_set_linked(layer, 1)				
		if rev==3:	# NOT-LINKED LAYER
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_item_set_linked(layer, 0)		
		if rev==5:	# GROUPS IND.
			group_new = pdb.gimp_layer_group_new(image)
			parent = pdb.gimp_item_get_parent(group_new)
			position = pdb.gimp_image_get_item_position(image, layer)
			pdb.gimp_image_insert_layer(image,group_new, None, 0) 
			group_new.name = "Group New"
			#for layer in apply_layers:
			pdb.gimp_image_reorder_item(image, layer, group_new, position)
			pdb.gimp_image_lower_item_to_bottom(image, layer)		
		if rev==6:	# UN-GROUP
			pdb.gimp_image_set_active_layer(image,layer)		
			position = pdb.gimp_image_get_item_position(image, layer)
			for child_layer in reversed(layer.children):
				pdb.gimp_image_reorder_item(image, child_layer, layer.parent, position)
			if pdb.gimp_item_is_group(layer):
				pdb.gimp_image_remove_layer(image, layer)
		if rev==7: # RAISES LAYER TO TOP (\"Reverse\"
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_image_raise_item_to_top(image, layer)
		if rev==8:	# RAISES LAYER TO BOTTOM (\"Reverse\"
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_image_lower_item_to_bottom(image, layer)
		if rev==9:  # CLEAR LAYERS
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_edit_clear(layer)		
		if rev==10:	# REMOVE LAYERS
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_image_remove_layer(image, layer)
		if rev==11: # Revers Visiible
			pdb.gimp_image_set_active_layer(image,layer)
			if layer.visible:		
				pdb.gimp_item_set_visible(layer, 0)
			else:				
				pdb.gimp_item_set_visible(layer, 1)	
		if rev==12:
			pdb.gimp_image_set_active_layer(image,layer)
			shuffle_layers(image,layer)

		in_rev=0 # no reverse
		if rev==13:	# sortLayersByName:
			sortLayers(image,in_rev, lambda apply_layers: apply_layers.name)
		if rev==14:	# sortLayersByWidth:
			sortLayers(image,in_rev, lambda apply_layers: apply_layers.width)
		if rev==15:	# sortLayersByHeight:
			sortLayers(image,in_rev, lambda apply_layers: apply_layers.height)
		if rev==16:	# sortLayersByArea:
			sortLayers(image,in_rev, lambda apply_layers: apply_layers.height*apply_layers.width)
		if rev==17:	# OPEN AS LAYERS IN NEW IMAGE
			pdb.gimp_image_set_active_layer(image,layer)		
			item_copy = pdb.gimp_layer_new_from_drawable(layer, image_create)
			pdb.gimp_image_insert_layer(image_create, item_copy, None, 0)
			item_copy.name = layer.name
			pdb.gimp_drawable_update(item_copy, 0, 0, item_copy.width, item_copy.height)
			pdb.gimp_image_lower_item_to_bottom(image_create, item_copy)		
		if rev==18:	# OPEN LAYERS AS NEW IMAGES	
			pdb.gimp_image_set_active_layer(image,layer)
			image_create = pdb.gimp_image_new(image.width, image.height, 0)
			display = pdb.gimp_display_new(image_create)
			item_copy = pdb.gimp_layer_new_from_drawable(layer, image_create)
			pdb.gimp_image_insert_layer(image_create, item_copy, None, 0)
			item_copy.name = layer.name
			pdb.gimp_drawable_update(item_copy, 0, 0, item_copy.width, item_copy.height)
			
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_MrQ_Switches_Layers",
	"Switches layers from selected sources to a specified target.",
	"Switches layers from selected sources to a specified target.",
	"MareroQ",
	"GPL",
	"2022",
	"Switches Layer...",
	"*",
	[
	(PF_IMAGE, "image",       "Input image", None),
	(PF_DRAWABLE, "layer", "Input drawable", None),	
	(PF_OPTION, "layergroup"," Select Source Layers: ", 0, [" ALL LAYERS "," VISIBLE LAYERS "," NOT-VISIBLE LAYERS "," LINKED LAYERS "," NOT-LINKED LAYERS "," ACTIVE ONLY "," ACTIVE AND ABOVE ", " ACTIVE AND BELOW "]),
	(PF_OPTION, "rev", " Target Result Layers: ", 0, [ " VISIBLE LAYER ", " NOT-VISIBLE LAYER ", " LINKED LAYER ", " NOT-LINKED LAYER "," GROUP IN ONE "," GROUP INDIVIDUALLY "," UNGROUP GROUP LAYER "," RAISES LAYER TO TOP\n ('Reverse') "," RAISES LAYER TO BOTTOM\n ('Reverse')"," CLEAR LAYERS "," REMOVE LAYERS ", " REVERSE VISIBILITY \n (visible → invisible | invisible → visible)"," SHUFFLE LAYERS "," SORT LAYERS By Name"," SORT LAYERS By Width"," SORT LAYERS By Height"," SORT LAYERS By Area"," OPEN LAYERS IN NEW ONE IMAGE"," OPEN LAYERS AS NEW IMAGES: "]),	
	
	],
	[],
	python_fu_MrQ_Switches_Layers, menu="<Layers>/layers-edit-attributes/")

main()
